@@ -1,9 +1,9 @@
 //
-// "$Id: Fl.cxx 9666 2012-08-16 20:59:36Z matt $"
+// "$Id$"
 //
 // Main event handling code for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2010 by Bill Spitzak and others.
+// Copyright 1998-2018 by Bill Spitzak and others.
 //
 // This library is free software. Distribution and use rights are outlined in
 // the file "COPYING" which should have been included with this file.  If this
@@ -39,8 +39,24 @@
 # endif
 #endif
 
+
+// STR #3454: We must #define FD_ISSET before we #include winsock2.h
+// to fix an undefined reference (__FD_ISSET). (AlbrechtS Feb 2018)
+// Other portions of the fix for STR #3454 are in src/Fl_win32.cxx.
+// This fix is only necessary for MinGW.
+
+#ifdef __MINGW32__
+static void * get_wsock_mod();
+typedef int(__stdcall *fl_wsk_fd_is_set_f)(unsigned int, void *);
+static fl_wsk_fd_is_set_f fl_wsk_fd_is_set = 0;
+
+#define FD_ISSET(S,SET) \
+  (get_wsock_mod() ? fl_wsk_fd_is_set(S, SET) : 0)
+#endif // __MINGW32__
+
 // recent versions of MinGW warn: "Please include winsock2.h before windows.h",
 // hence we must include winsock2.h before FL/Fl.H (A.S. Dec. 2010, IMM May 2011)
+
 #if defined(WIN32) && !defined(__CYGWIN__)
 #  include <winsock2.h>
 #endif
@@ -51,7 +67,6 @@
 #include <FL/x.H>
 
 #include <ctype.h>
-#include <stdio.h>
 #include <stdlib.h>
 #include "flstring.h"
 
@@ -67,21 +82,38 @@ void fl_cleanup_pens(void);
 void fl_release_dc(HWND,HDC);
 void fl_cleanup_dc_list(void);
 #elif defined(__APPLE__)
-extern double fl_mac_flush_and_wait(double time_to_wait, char in_idle);
+extern double fl_mac_flush_and_wait(double time_to_wait);
 #endif // WIN32
 
 //
 // Globals...
 //
+
+// Pointers you can use to change FLTK to a foreign language.
+// Note: Similar pointers are defined in FL/fl_ask.H and src/fl_ask.cxx
+#if !defined(__APPLE__) || defined(FL_DOXYGEN)
+  const char* fl_local_alt   = "Alt";	///< string pointer used in shortcuts, you can change it to another language
+  const char* fl_local_ctrl  = "Ctrl";	///< string pointer used in shortcuts, you can change it to another language
+  const char* fl_local_meta  = "Meta";	///< string pointer used in shortcuts, you can change it to another language
+  const char* fl_local_shift = "Shift";	///< string pointer used in shortcuts, you can change it to another language
+#else
+  const char* fl_local_alt   = "\xe2\x8c\xa5\\"; // U+2325 (option key)
+  const char* fl_local_ctrl  = "\xe2\x8c\x83\\"; // U+2303 (up arrowhead)
+  const char* fl_local_meta  = "\xe2\x8c\x98\\"; // U+2318 (place of interest sign)
+  const char* fl_local_shift = "\xe2\x87\xa7\\"; // U+21E7 (upwards white arrow)
+#endif
+
+// Apple App Menu
 #if defined(__APPLE__) || defined(FL_DOXYGEN)
-const char *Fl_Mac_App_Menu::about = "About ";
+const char *Fl_Mac_App_Menu::about = "About %@";
 const char *Fl_Mac_App_Menu::print = "Print Front Window";
 const char *Fl_Mac_App_Menu::services = "Services";
-const char *Fl_Mac_App_Menu::hide = "Hide ";
+const char *Fl_Mac_App_Menu::hide = "Hide %@";
 const char *Fl_Mac_App_Menu::hide_others = "Hide Others";
 const char *Fl_Mac_App_Menu::show = "Show All";
-const char *Fl_Mac_App_Menu::quit = "Quit ";
+const char *Fl_Mac_App_Menu::quit = "Quit %@";
 #endif // __APPLE__
+
 #ifndef FL_DOXYGEN
 Fl_Widget	*Fl::belowmouse_,
 		*Fl::pushed_,
@@ -104,6 +136,8 @@ int		Fl::damage_,
 
 char		*Fl::e_text = (char *)"";
 int		Fl::e_length;
+const char*	Fl::e_clipboard_type = "";
+void *		Fl::e_clipboard_data = NULL;
 
 Fl_Event_Dispatch Fl::e_dispatch = 0;
 
@@ -111,26 +145,46 @@ unsigned char   Fl::options_[] = { 0, 0 };
 unsigned char   Fl::options_read_ = 0;
 
 
-Fl_Window *fl_xfocus;	// which window X thinks has focus
+Fl_Window *fl_xfocus = NULL;	// which window X thinks has focus
 Fl_Window *fl_xmousewin;// which window X thinks has FL_ENTER
 Fl_Window *Fl::grab_;	// most recent Fl::grab()
 Fl_Window *Fl::modal_;	// topmost modal() window
 
 #endif // FL_DOXYGEN
 
+char const * const Fl::clipboard_plain_text = "text/plain";
+char const * const Fl::clipboard_image = "image";
+
 //
 // 'Fl::version()' - Return the API version number...
 //
 
-double
 /**
   Returns the compiled-in value of the FL_VERSION constant. This
   is useful for checking the version of a shared library.
+
+  \deprecated	Use int Fl::api_version() instead.
 */
-Fl::version() {
+double Fl::version() {
   return FL_VERSION;
 }
 
+/**
+  Returns the compiled-in value of the FL_API_VERSION constant. This
+  is useful for checking the version of a shared library.
+*/
+int Fl::api_version() {
+  return FL_API_VERSION;
+}
+
+/**
+  Returns the compiled-in value of the FL_ABI_VERSION constant. This
+  is useful for checking the version of a shared library.
+*/
+int  Fl::abi_version() {
+  return FL_ABI_VERSION;
+}
+
 /**
   Gets the default scrollbar size used by
   Fl_Browser_,
@@ -220,7 +274,7 @@ int Fl::event_inside(const Fl_Widget *o) /*const*/ {
 
 #elif defined(__APPLE__)
 
-// implementation in Fl_mac.cxx
+// implementation in Fl_cocoa.mm (was Fl_mac.cxx)
 
 #else
 
@@ -430,10 +484,74 @@ static void run_checks()
   }
 }
 
-#ifndef WIN32
+#if !defined(WIN32) && !defined(__APPLE__)
 static char in_idle;
 #endif
 
+////////////////////////////////////////////////////////////////
+// Clipboard notifications
+
+struct Clipboard_Notify {
+  Fl_Clipboard_Notify_Handler handler;
+  void *data;
+  struct Clipboard_Notify *next;
+};
+
+static struct Clipboard_Notify *clip_notify_list = NULL;
+
+extern void fl_clipboard_notify_change(); // in Fl_<platform>.cxx
+
+void Fl::add_clipboard_notify(Fl_Clipboard_Notify_Handler h, void *data) {
+  struct Clipboard_Notify *node;
+
+  remove_clipboard_notify(h);
+
+  node = new Clipboard_Notify;
+
+  node->handler = h;
+  node->data = data;
+  node->next = clip_notify_list;
+
+  clip_notify_list = node;
+
+  fl_clipboard_notify_change();
+}
+
+void Fl::remove_clipboard_notify(Fl_Clipboard_Notify_Handler h) {
+  struct Clipboard_Notify *node, **prev;
+
+  node = clip_notify_list;
+  prev = &clip_notify_list;
+  while (node != NULL) {
+    if (node->handler == h) {
+      *prev = node->next;
+      delete node;
+
+      fl_clipboard_notify_change();
+
+      return;
+    }
+
+    prev = &node->next;
+    node = node->next;
+  }
+}
+
+bool fl_clipboard_notify_empty(void) {
+  return clip_notify_list == NULL;
+}
+
+void fl_trigger_clipboard_notify(int source) {
+  struct Clipboard_Notify *node, *next;
+
+  node = clip_notify_list;
+  while (node != NULL) {
+    next = node->next;
+    node->handler(source, node->data);
+    node = next;
+  }
+}
+
 ////////////////////////////////////////////////////////////////
 // wait/run/check/ready:
 
@@ -456,16 +574,7 @@ double Fl::wait(double time_to_wait) {
 #elif defined(__APPLE__)
 
   run_checks();
-  if (idle) {
-    if (!in_idle) {
-      in_idle = 1;
-      idle();
-      in_idle = 0;
-    }
-    // the idle function may turn off idle, we can then wait:
-    if (idle) time_to_wait = 0.0;
-  }
-  return fl_mac_flush_and_wait(time_to_wait, in_idle);
+  return fl_mac_flush_and_wait(time_to_wait);
 
 #else
 
@@ -530,45 +639,6 @@ int Fl::run() {
   return 0;
 }
 
-#ifdef WIN32
-
-// Function to initialize COM/OLE for usage. This must be done only once.
-// We define a flag to register whether we called it:
-static char oleInitialized = 0;
-
-// This calls the Windows function OleInitialize() exactly once.
-void fl_OleInitialize() {
-  if (!oleInitialized) {
-    OleInitialize(0L);
-    oleInitialized = 1;
-  }
-}
-
-// This calls the Windows function OleUninitialize() only, if
-// OleInitialize has been called before.
-void fl_OleUninitialize() {
-  if (oleInitialized) {
-    OleUninitialize();
-    oleInitialized = 0;
-  }
-}
-
-class Fl_Win32_At_Exit {
-public:
-  Fl_Win32_At_Exit() { }
-  ~Fl_Win32_At_Exit() {
-    fl_free_fonts();        // do some WIN32 cleanup
-    fl_cleanup_pens();
-    fl_OleUninitialize();
-    fl_brush_action(1);
-    fl_cleanup_dc_list();
-  }
-};
-static Fl_Win32_At_Exit win32_at_exit;
-#endif
-
-
-
 /**
   Waits until "something happens" and then returns.  Call this
   repeatedly to "run" your program.  You can also check what happened
@@ -660,20 +730,13 @@ Fl_X* Fl_X::first;
 Fl_Window* fl_find(Window xid) {
   Fl_X *window;
   for (Fl_X **pp = &Fl_X::first; (window = *pp); pp = &window->next)
-#if defined(WIN32) || defined(USE_X11)
-   if (window->xid == xid)
-#elif defined(__APPLE_QUARTZ__)
-   if (window->xid == xid && !window->w->window())
-#else
-# error unsupported platform
-#endif // __APPLE__
-	{
+   if (window->xid == xid) {
       if (window != Fl_X::first && !Fl::modal()) {
-	// make this window be first to speed up searches
-	// this is not done if modal is true to avoid messing up modal stack
-	*pp = window->next;
-	window->next = Fl_X::first;
-	Fl_X::first = window;
+        // make this window be first to speed up searches
+        // this is not done if modal is true to avoid messing up modal stack
+        *pp = window->next;
+        window->next = Fl_X::first;
+        Fl_X::first = window;
       }
       return window->w;
     }
@@ -781,7 +844,7 @@ static handler_link *handlers = 0;
   - \ref FL_SCREEN_CONFIGURATION_CHANGED events.
     Under X11, this event requires the libXrandr.so shared library to be
     loadable at run-time and the X server to implement the RandR extension.
-  - \ref FL_FULLSCREEN events sent to a window that enters of leaves
+  - \ref FL_FULLSCREEN events sent to a window that enters or leaves
     fullscreen mode.
   - System events that FLTK does not recognize.  See fl_xevent.
   - \e Some other events when the widget FLTK selected returns
@@ -808,7 +871,7 @@ void Fl::remove_handler(Fl_Event_Handler ha) {
   handler_link *l, *p;
 
   // Search for the handler in the list...
-  for (l = handlers, p = 0; l && l->handle != ha; p = l, l = l->next);
+  for (l = handlers, p = 0; l && l->handle != ha; p = l, l = l->next) {/*empty*/}
 
   if (l) {
     // Found it, so remove it from the list...
@@ -828,6 +891,83 @@ static int send_handlers(int e) {
   return 0;
 }
 
+
+////////////////////////////////////////////////////////////////
+// System event handlers:
+
+
+struct system_handler_link {
+  Fl_System_Handler handle;
+  void *data;
+  system_handler_link *next;
+};
+
+
+static system_handler_link *sys_handlers = 0;
+
+
+/**
+ \brief Install a function to intercept system events.
+
+ FLTK calls each of these functions as soon as a new system event is
+ received. The processing will stop at the first function to return
+ non-zero. If all functions return zero then the event is passed on
+ for normal handling by FLTK.
+
+ Each function will be called with a pointer to the system event as
+ the first argument and \p data as the second argument. The system
+ event pointer will always be void *, but will point to different
+ objects depending on the platform:
+   - X11: XEvent
+   - Windows: MSG
+   - OS X: NSEvent
+
+ \param ha The event handler function to register
+ \param data User data to include on each call
+
+ \see Fl::remove_system_handler(Fl_System_Handler)
+*/
+void Fl::add_system_handler(Fl_System_Handler ha, void *data) {
+  system_handler_link *l = new system_handler_link;
+  l->handle = ha;
+  l->data = data;
+  l->next = sys_handlers;
+  sys_handlers = l;
+}
+
+
+/**
+ Removes a previously added system event handler.
+
+ \param ha The event handler function to remove
+
+ \see Fl::add_system_handler(Fl_System_Handler)
+*/
+void Fl::remove_system_handler(Fl_System_Handler ha) {
+  system_handler_link *l, *p;
+
+  // Search for the handler in the list...
+  for (l = sys_handlers, p = 0; l && l->handle != ha; p = l, l = l->next);
+
+  if (l) {
+    // Found it, so remove it from the list...
+    if (p) p->next = l->next;
+    else sys_handlers = l->next;
+
+    // And free the record...
+    delete l;
+  }
+}
+
+int fl_send_system_handlers(void *e) {
+  for (const system_handler_link *hl = sys_handlers; hl; hl = hl->next) {
+    if (hl->handle(e, hl->data))
+      return 1;
+  }
+  return 0;
+}
+
+
 ////////////////////////////////////////////////////////////////
 
 Fl_Widget* fl_oldfocus; // kludge for Fl_Group...
@@ -862,10 +1002,18 @@ void Fl::focus(Fl_Widget *o) {
 	if (fl_xfocus != win) {
 	  Fl_X *x = Fl_X::i(win);
 	  if (x) x->set_key_window();
-	  }
+	}
+#elif defined(USE_X11)
+	if (fl_xfocus != win) {
+	  Fl_X *x = Fl_X::i(win);
+	  if (!Fl_X::ewmh_supported())
+	    win->show(); // Old WMs, XMapRaised
+	  else if (x) // New WMs use the NETWM attribute:
+	    Fl_X::activate_window(x->xid);
+	}
 #endif
 	fl_xfocus = win;
-	}
+      }
     }
     // take focus from the old focused window
     fl_oldfocus = 0;
@@ -1018,10 +1166,41 @@ void fl_throw_focus(Fl_Widget *o) {
 
 ////////////////////////////////////////////////////////////////
 
+// Find the first active_r() widget, starting at the widget wi and
+// walking up the widget hierarchy to the top level window.
+//
+// In other words: find_active() returns an active group that contains
+// the inactive widget and all inactive parent groups.
+//
+// This is used to send FL_SHORTCUT events to the Fl::belowmouse() widget
+// in case the target widget itself is inactive_r(). In this case the event
+// is sent to the first active_r() parent.
+//
+// This prevents sending events to inactive widgets that might get the
+// input focus otherwise. The search is fast and light and avoids calling
+// inactive_r() multiple times.
+// See STR #3216.
+//
+// Returns: first active_r() widget "above" the widget wi or NULL if
+// no widget is active. May return the top level window.
+
+static Fl_Widget *find_active(Fl_Widget *wi) {
+  Fl_Widget *found = 0;
+  for (; wi; wi = wi->parent()) {
+    if (wi->active()) {
+      if (!found) found = wi;
+    }
+    else found = 0;
+  }
+  return found;
+}
+
+////////////////////////////////////////////////////////////////
+
 // Call to->handle(), but first replace the mouse x/y with the correct
 // values to account for nested windows. 'window' is the outermost
 // window the event was posted to by the system:
-static int send(int event, Fl_Widget* to, Fl_Window* window) {
+static int send_event(int event, Fl_Widget* to, Fl_Window* window) {
   int dx, dy;
   int old_event = Fl::e_number;
   if (window) {
@@ -1163,7 +1342,7 @@ int Fl::handle_(int e, Fl_Window* window)
     else if (modal() && wi != modal()) return 0;
     pushed_ = wi;
     Fl_Tooltip::current(wi);
-    if (send(e, wi, window)) return 1;
+    if (send_event(e, wi, window)) return 1;
     // raise windows that are clicked on:
     window->show();
     return 1;
@@ -1207,11 +1386,11 @@ int Fl::handle_(int e, Fl_Window* window)
 	  Fl::event_y_root() >= tooltip->y() && Fl::event_y_root() < tooltip->y() + tooltip->h() );
 	}
 	// if inside, send event to tooltip window instead of background window
-	if (inside) ret = send(e, tooltip, window);
-	else ret = (wi && send(e, wi, window));
+	if (inside) ret = send_event(e, tooltip, window);
+	else ret = (wi && send_event(e, wi, window));
       } else
 #endif
-      ret = (wi && send(e, wi, window));
+      ret = (wi && send_event(e, wi, window));
    if (pbm != belowmouse()) {
 #ifdef DEBUG
       printf("Fl::handle(e=%d, window=%p);\n", e, window);
@@ -1231,7 +1410,7 @@ int Fl::handle_(int e, Fl_Window* window)
       wi = pushed();
       pushed_ = 0; // must be zero before callback is done!
     } else if (modal() && wi != modal()) return 0;
-    int r = send(e, wi, window);
+    int r = send_event(e, wi, window);
     fl_fix_focus();
     return r;}
 
@@ -1253,7 +1432,7 @@ int Fl::handle_(int e, Fl_Window* window)
     // a KEYUP there. I believe that the current solution is
     // "close enough".
     for (wi = grab() ? grab() : focus(); wi; wi = wi->parent())
-      if (send(FL_KEYUP, wi, window)) return 1;
+      if (send_event(FL_KEYUP, wi, window)) return 1;
     return 0;
 
   case FL_KEYBOARD:
@@ -1267,7 +1446,7 @@ int Fl::handle_(int e, Fl_Window* window)
 
     // Try it as keystroke, sending it to focus and all parents:
     for (wi = grab() ? grab() : focus(); wi; wi = wi->parent())
-      if (send(FL_KEYBOARD, wi, window)) return 1;
+      if (send_event(FL_KEYBOARD, wi, window)) return 1;
 
     // recursive call to try shortcut:
     if (handle(FL_SHORTCUT, window)) return 1;
@@ -1283,16 +1462,16 @@ int Fl::handle_(int e, Fl_Window* window)
     if (grab()) {wi = grab(); break;} // send it to grab window
 
     // Try it as shortcut, sending to mouse widget and all parents:
-    wi = belowmouse();
+    wi = find_active(belowmouse()); // STR #3216
     if (!wi) {
       wi = modal();
       if (!wi) wi = window;
     } else if (wi->window() != first_window()) {
-      if (send(FL_SHORTCUT, first_window(), first_window())) return 1;
+      if (send_event(FL_SHORTCUT, first_window(), first_window())) return 1;
     }
 
     for (; wi; wi = wi->parent()) {
-      if (send(FL_SHORTCUT, wi, wi->window())) return 1;
+      if (send_event(FL_SHORTCUT, wi, wi->window())) return 1;
     }
 
     // try using add_handle() functions:
@@ -1334,19 +1513,19 @@ int Fl::handle_(int e, Fl_Window* window)
 
     // Try sending it to the "grab" first
     if (grab() && grab()!=modal() && grab()!=window) {
-      if (send(FL_MOUSEWHEEL, grab(), window)) return 1;
+      if (send_event(FL_MOUSEWHEEL, grab(), window)) return 1;
     }
     // Now try sending it to the "modal" window
     if (modal()) {
-      send(FL_MOUSEWHEEL, modal(), window);
+      send_event(FL_MOUSEWHEEL, modal(), window);
       return 1;
     }
     // Finally try sending it to the window, the event occured in
-    if (send(FL_MOUSEWHEEL, window, window)) return 1;
+    if (send_event(FL_MOUSEWHEEL, window, window)) return 1;
   default:
     break;
   }
-  if (wi && send(e, wi, window)) {
+  if (wi && send_event(e, wi, window)) {
     dnd_flag = 0;
     return 1;
   }
@@ -1357,11 +1536,38 @@ int Fl::handle_(int e, Fl_Window* window)
 ////////////////////////////////////////////////////////////////
 // hide() destroys the X window, it does not do unmap!
 
-#if !defined(WIN32) && USE_XFT
+#if defined(WIN32)
+extern void fl_clipboard_notify_retarget(HWND wnd);
+extern void fl_update_clipboard(void);
+#elif USE_XFT
 extern void fl_destroy_xft_draw(Window);
 #endif
 
 void Fl_Window::hide() {
+#ifdef WIN32
+  // STR#3079: if there remains a window and a non-modal window, and the window is deleted,
+  // the app remains running without any apparent window.
+  // Bug mechanism: hiding an owner window unmaps the owned (non-modal) window(s)
+  // but does not delete it(them) in FLTK.
+  // Fix for it: 
+  // when hiding a window, build list of windows it owns, and do hide/show on them.
+  int count = 0;
+  Fl_Window *win, **doit = NULL;
+  for (win = Fl::first_window(); win && i; win = Fl::next_window(win)) {
+    if (win->non_modal() && GetWindow(fl_xid(win), GW_OWNER) == i->xid) {
+      count++;
+    }
+  }
+  if (count) {
+    doit = new Fl_Window*[count];
+    count = 0;
+    for (win = Fl::first_window(); win && i; win = Fl::next_window(win)) {
+      if (win->non_modal() && GetWindow(fl_xid(win), GW_OWNER) == i->xid) {
+	doit[count++] = win;
+      }
+    }
+  }
+#endif
   clear_visible();
 
   if (!shown()) return;
@@ -1372,7 +1578,6 @@ void Fl_Window::hide() {
   for (; *pp != ip; pp = &(*pp)->next) if (!*pp) return;
   *pp = ip->next;
 #ifdef __APPLE__
-  ip->unlink();
   // MacOS X manages a single pointer per application. Make sure that hiding
   // a toplevel window will not leave us with some random pointer shape, or
   // worst case, an invisible pointer
@@ -1402,16 +1607,14 @@ void Fl_Window::hide() {
   handle(FL_HIDE);
 
 #if defined(WIN32)
+  // make sure any custom icons get freed
+  icons(NULL, 0);
   // this little trick keeps the current clipboard alive, even if we are about
   // to destroy the window that owns the selection.
-  if (GetClipboardOwner()==ip->xid) {
-    Fl_Window *w1 = Fl::first_window();
-    if (w1 && OpenClipboard(fl_xid(w1))) {
-      EmptyClipboard();
-      SetClipboardData(CF_TEXT, NULL);
-      CloseClipboard();
-    }
-  }
+  if (GetClipboardOwner()==ip->xid)
+    fl_update_clipboard();
+  // Make sure we unlink this window from the clipboard chain
+  fl_clipboard_notify_retarget(ip->xid);
   // Send a message to myself so that I'll get out of the event loop...
   PostMessage(ip->xid, WM_APP, 0, 0);
   if (ip->private_dc) fl_release_dc(ip->xid, ip->private_dc);
@@ -1425,7 +1628,7 @@ void Fl_Window::hide() {
     }
 #elif defined(__APPLE_QUARTZ__)
   Fl_X::q_release_context(ip);
-  if ( ip->xid == fl_window && !parent() )
+  if ( ip->xid == fl_window )
     fl_window = 0;
 #endif
 
@@ -1445,6 +1648,16 @@ void Fl_Window::hide() {
     ShowWindow(p, SW_SHOWNA);
   }
   XDestroyWindow(fl_display, ip->xid);
+  // end of fix for STR#3079
+  if (count) {
+    int ii;
+    for (ii = 0; ii < count; ii++)  doit[ii]->hide();
+    for (ii = 0; ii < count; ii++)  {
+      if (ii != 0) doit[0]->show(); // Fix for STR#3165
+      doit[ii]->show();
+    }
+    delete[] doit;
+  }
 #elif defined(__APPLE_QUARTZ__)
   ip->destroy();
 #else
@@ -1459,12 +1672,6 @@ void Fl_Window::hide() {
   delete ip;
 }
 
-Fl_Window::~Fl_Window() {
-  hide();
-  if (xclass_) {
-    free(xclass_);
-  }
-}
 
 // FL_SHOW and FL_HIDE are called whenever the visibility of this widget
 // or any parent changes.  We must correctly map/unmap the system's window.
@@ -1551,12 +1758,23 @@ void Fl::selection(Fl_Widget &owner, const char* text, int len) {
 
 /** Backward compatibility only.
   This calls Fl::paste(receiver, 0);
-  \see Fl::paste(Fl_Widget &receiver, int clipboard)
+  \see Fl::paste(Fl_Widget &receiver, int clipboard, const char* type)
 */
 void Fl::paste(Fl_Widget &receiver) {
   Fl::paste(receiver, 0);
 }
+#if FLTK_ABI_VERSION >= 10303
+#elif !defined(FL_DOXYGEN)
+void Fl::paste(Fl_Widget &receiver, int source)
+{
+  Fl::paste(receiver, source, Fl::clipboard_plain_text);
+}
 
+void Fl::copy(const char* stuff, int len, int destination) {
+  Fl::copy(stuff, len, destination, Fl::clipboard_plain_text);
+}
+
+#endif
 ////////////////////////////////////////////////////////////////
 
 #include <FL/fl_draw.H>
@@ -1696,6 +1914,7 @@ void Fl_Widget::damage(uchar fl, int X, int Y, int W, int H) {
   Fl::damage(FL_DAMAGE_CHILD);
 }
 void Fl_Window::flush() {
+  if (!shown()) return;
   make_current();
 //if (damage() == FL_DAMAGE_EXPOSE && can_boxcheat(box())) fl_boxcheat = this;
   fl_clip_region(i->region); i->region = 0;
@@ -1729,7 +1948,13 @@ static Fl_Widget	**dwidgets = 0;
   When deleting groups or windows, you must only delete the group or
   window widget and not the individual child widgets.
 
-  \since FLTK 1.3 it is not necessary to remove widgets from their parent
+  \since FLTK 1.3.4 the widget will be hidden immediately, but the actual
+  destruction will be delayed until the event loop is finished. Up to
+  FLTK 1.3.3 windows wouldn't be hidden before the event loop was done,
+  hence you had to hide() a window in your window close callback if
+  you called Fl::delete_widget() to destroy (and hide) the window.
+
+  \since FLTK 1.3.0 it is not necessary to remove widgets from their parent
   groups or windows before calling this, because it will be done in the
   widget's destructor, but it is not a failure to do this nevertheless.
 
@@ -1740,8 +1965,13 @@ static Fl_Widget	**dwidgets = 0;
 */
 void Fl::delete_widget(Fl_Widget *wi) {
   if (!wi) return;
-  
-  // don;t add the same widget twice
+
+  // if the widget is shown(), hide() it (FLTK 1.3.4)
+  if (wi->visible_r()) wi->hide();
+  Fl_Window *win = wi->as_window();
+  if (win && win->shown()) win->hide(); // case of iconified window
+
+  // don't add the same widget twice to the widget delete list
   for (int i = 0; i < num_dwidgets; i++) {
     if (dwidgets[i]==wi) return;
   }
@@ -1931,6 +2161,14 @@ void Fl::clear_widget_pointer(Fl_Widget const *w)
  There should be an application that manages options system wide, per user, and
  per application.
 
+ Example:
+ \code
+     if ( Fl::option(Fl::OPTION_ARROW_FOCUS) )
+         { ..on..  }
+     else
+         { ..off..  }
+ \endcode
+
  \note As of FLTK 1.3.0, options can be managed within fluid, using the menu
  <i>Edit/Global FLTK Settings</i>.
 
@@ -1960,6 +2198,8 @@ bool Fl::option(Fl_Option opt)
       options_[OPTION_DND_TEXT] = tmp;
       opt_prefs.get("ShowTooltips", tmp, 1);                    // default: on
       options_[OPTION_SHOW_TOOLTIPS] = tmp;
+      opt_prefs.get("FNFCUsesGTK", tmp, 1);                    // default: on
+      options_[OPTION_FNFC_USES_GTK] = tmp;
     }
     { // next, check the user preferences
       // override system options only, if the option is set ( >= 0 )
@@ -1977,6 +2217,8 @@ bool Fl::option(Fl_Option opt)
       if (tmp >= 0) options_[OPTION_DND_TEXT] = tmp;
       opt_prefs.get("ShowTooltips", tmp, -1);
       if (tmp >= 0) options_[OPTION_SHOW_TOOLTIPS] = tmp;
+      opt_prefs.get("FNFCUsesGTK", tmp, -1);
+      if (tmp >= 0) options_[OPTION_FNFC_USES_GTK] = tmp;
     }
     { // now, if the developer has registered this app, we could as for per-application preferences
     }
@@ -1992,6 +2234,12 @@ bool Fl::option(Fl_Option opt)
 
  This function does not change any system or user settings.
 
+ Example:
+ \code
+     Fl::option(Fl::OPTION_ARROW_FOCUS, true);     // on
+     Fl::option(Fl::OPTION_ARROW_FOCUS, false);    // off
+ \endcode
+
  \param opt which option
  \param val set to true or false
  \see enum Fl::Fl_Option
@@ -2028,7 +2276,7 @@ Fl_Widget_Tracker::~Fl_Widget_Tracker()
   Fl::release_widget_pointer(wp_); // remove pointer from watch list
 }
 
-
+int Fl::use_high_res_GL_ = 0;
 //
-// End of "$Id: Fl.cxx 9666 2012-08-16 20:59:36Z matt $".
+// End of "$Id$".
 //
