@@ -1,5 +1,5 @@
 //
-// "$Id: Fl_Copy_Surface.cxx 11898 2016-08-27 15:17:02Z manolo $"
+// "$Id$"
 //
 // Copy-to-clipboard code for the Fast Light Tool Kit (FLTK).
 //
@@ -225,22 +225,26 @@ void Fl_Copy_Surface::draw_decorated_window(Fl_Window* win, int delta_x, int del
     void *layer = Fl_X::get_titlebar_layer(win);
     if (layer) {
       CGColorSpaceRef cspace = CGColorSpaceCreateDeviceRGB();
-      // for unknown reason, rendering the layer to the Fl_Copy_Surface pdf graphics context does not work;
-      // we use an auxiliary bitmap context
-      CGContextRef auxgc = CGBitmapContextCreate(NULL, win->w(), bt, 8, 0, cspace, kCGImageAlphaPremultipliedLast);
-      CGColorSpaceRelease(cspace);
-      CGContextTranslateCTM(auxgc, 0, bt);
-      CGContextScaleCTM(auxgc, 1, -1);
-      Fl_X::draw_layer_to_context(layer, auxgc, win->w(), bt);
-      Fl_RGB_Image *image = new Fl_RGB_Image((const uchar*)CGBitmapContextGetData(auxgc), win->w(), bt, 4,
-                                             CGBitmapContextGetBytesPerRow(auxgc)); // 10.2
-      image->draw(0, 0);
-      delete image;
-      CGContextRelease(auxgc);
+      if (fl_mac_os_version < 101300) {
+        // for unknown reason, rendering the layer to the Fl_Copy_Surface pdf graphics context does not work;
+        // we use an auxiliary bitmap context
+        CGContextRef auxgc = CGBitmapContextCreate(NULL, win->w(), bt, 8, 0, cspace, kCGImageAlphaPremultipliedLast);
+        CGColorSpaceRelease(cspace);
+        CGContextTranslateCTM(auxgc, 0, bt);
+        CGContextScaleCTM(auxgc, 1, -1);
+        Fl_X::draw_layer_to_context(layer, auxgc, win->w(), bt);
+        Fl_RGB_Image *image = new Fl_RGB_Image((const uchar*)CGBitmapContextGetData(auxgc), win->w(), bt, 4,
+                                               CGBitmapContextGetBytesPerRow(auxgc)); // 10.2
+        image->draw(0, 0);
+        delete image;
+        CGContextRelease(auxgc);
+      } else {
+        Fl_X::draw_layer_to_context(layer, gc, win->w(), bt);
+      }
     } else {
       CGImageRef img = Fl_X::CGImage_from_window_rect(win, 0, -bt, win->w(), bt);
       CGContextDrawImage(gc, CGRectMake(0, 0, win->w(), bt), img);
-      CFRelease(img);
+      CGImageRelease(img);
     }
     CGContextRestoreGState(gc);
   }
@@ -399,5 +403,5 @@ const char *Fl_Xlib_Surface_::class_id = "Fl_Xlib_Surface_";
 #endif
 
 //
-// End of "$Id: Fl_Copy_Surface.cxx 11898 2016-08-27 15:17:02Z manolo $".
+// End of "$Id$".
 //
