@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_GIF_Image.cxx 8864 2011-07-19 04:49:30Z greg.ercolano $"
+// "$Id$"
 //
 // Fl_GIF_Image routines.
 //
-// Copyright 1997-2010 by Bill Spitzak and others.
+// Copyright 1997-2019 by Bill Spitzak and others.
 //
 // This library is free software. Distribution and use rights are outlined in
 // the file "COPYING" which should have been included with this file.  If this
@@ -19,6 +19,16 @@
 //
 //
 
+//
+// Reference: GIF89a Specification (links valid as of Jan 05, 2019):
+//
+// "GRAPHICS INTERCHANGE FORMAT(sm), Version 89a" (authoritative):
+// https://www.w3.org/Graphics/GIF/spec-gif89a.txt
+//
+// HTML version (non-authoritative):
+// https://web.archive.org/web/20160304075538/http://qalle.net/gif89a.php
+//
+
 //
 // Include necessary header files...
 //
@@ -70,28 +80,37 @@ typedef unsigned char uchar;
 #define NEXTBYTE (uchar)getc(GifFile)
 #define GETSHORT(var) var = NEXTBYTE; var += NEXTBYTE << 8
 
-/**  
-  The constructor loads the named GIF image.
-  <P>The inherited destructor free all memory and server resources that are used by
-  the image.
-*/
+/**
+ The constructor loads the named GIF image.
+
+ The destructor frees all memory and server resources that are used by
+ the image.
+
+ Use Fl_Image::fail() to check if Fl_GIF_Image failed to load. fail() returns
+ ERR_FILE_ACCESS if the file could not be opened or read, ERR_FORMAT if the
+ GIF format could not be decoded, and ERR_NO_IMAGE if the image could not
+ be loaded for another reason.
+ */
 Fl_GIF_Image::Fl_GIF_Image(const char *infname) : Fl_Pixmap((char *const*)0) {
   FILE *GifFile;	// File to read
   char **new_data;	// Data array
 
   if ((GifFile = fl_fopen(infname, "rb")) == NULL) {
     Fl::error("Fl_GIF_Image: Unable to open %s!", infname);
+    ld(ERR_FILE_ACCESS);
     return;
   }
 
   {char b[6];
   if (fread(b,1,6,GifFile)<6) {
     fclose(GifFile);
+    ld(ERR_FILE_ACCESS);
     return; /* quit on eof */
   }
   if (b[0]!='G' || b[1]!='I' || b[2] != 'F') {
     fclose(GifFile);
     Fl::error("Fl_GIF_Image: %s is not a GIF file.\n", infname);
+    ld(ERR_FORMAT);
     return;
   }
   if (b[3]!='8' || b[4]>'9' || b[5]!= 'a')
@@ -104,7 +123,12 @@ Fl_GIF_Image::Fl_GIF_Image(const char *infname) : Fl_Pixmap((char *const*)0) {
   uchar ch = NEXTBYTE;
   char HasColormap = ((ch & 0x80) != 0);
   int BitsPerPixel = (ch & 7) + 1;
-  int ColorMapSize = 1 << BitsPerPixel;
+  int ColorMapSize;
+  if (HasColormap) {
+    ColorMapSize = 2 << (ch & 7);
+  } else {
+    ColorMapSize = 0;
+  }
   // int OriginalResolution = ((ch>>4)&7)+1;
   // int SortedTable = (ch&8)!=0;
   ch = NEXTBYTE; // Background Color index
@@ -120,10 +144,6 @@ Fl_GIF_Image::Fl_GIF_Image(const char *infname) : Fl_Pixmap((char *const*)0) {
       Green[i] = NEXTBYTE;
       Blue[i] = NEXTBYTE;
     }
-  } else {
-    Fl::warning("%s does not have a colormap.", infname);
-    for (int i = 0; i < ColorMapSize; i++)
-      Red[i] = Green[i] = Blue[i] = (uchar)(255 * i / (ColorMapSize-1));
   }
 
   int CodeSize;		/* Code size, init from GIF header, increases... */
@@ -135,6 +155,7 @@ Fl_GIF_Image::Fl_GIF_Image(const char *infname) : Fl_Pixmap((char *const*)0) {
     if (i<0) {
       fclose(GifFile);
       Fl::error("Fl_GIF_Image: %s - unexpected EOF",infname); 
+      w(0); h(0); d(0); ld(ERR_FORMAT);
       return;
     }
     int blocklen;
@@ -169,10 +190,10 @@ Fl_GIF_Image::Fl_GIF_Image(const char *infname) : Fl_Pixmap((char *const*)0) {
       GETSHORT(Height);
       ch = NEXTBYTE;
       Interlace = ((ch & 0x40) != 0);
-      if (ch&0x80) { 
-	// read local color map
-	int n = 2<<(ch&7);
-	for (i=0; i < n; i++) {	
+      if (ch & 0x80) { // image has local color table
+	BitsPerPixel = (ch & 7) + 1;
+	ColorMapSize = 2 << (ch & 7);
+	for (i=0; i < ColorMapSize; i++) {
 	  Red[i] = NEXTBYTE;
 	  Green[i] = NEXTBYTE;
 	  Blue[i] = NEXTBYTE;
@@ -196,6 +217,27 @@ Fl_GIF_Image::Fl_GIF_Image(const char *infname) : Fl_Pixmap((char *const*)0) {
     ColorMapSize = 1 << BitsPerPixel;
   }
 
+  // Fix images w/o color table. The standard allows this and lets the
+  // decoder choose a default color table. The standard recommends the
+  // first two color table entries should be black and white.
+
+  if (ColorMapSize == 0) { // no global and no local color table
+    Fl::warning("%s does not have a color table, using default.\n", infname);
+    BitsPerPixel = CodeSize - 1;
+    ColorMapSize = 1 << BitsPerPixel;
+    Red[0] = Green[0] = Blue[0] = 0;	// black
+    Red[1] = Green[1] = Blue[1] = 255;	// white
+    for (int i = 2; i < ColorMapSize; i++) {
+      Red[i] = Green[i] = Blue[i] = (uchar)(255 * i / (ColorMapSize - 1));
+    }
+#if (0)
+    // fill color table to maximum size
+    for (int i = ColorMapSize; i < 256; i++) {
+      Red[i] = Green[i] = Blue[i] = 0; // black
+    }
+#endif
+  }
+
   uchar *Image = new uchar[Width*Height];
 
   int YC = 0, Pass = 0; /* Used to de-interlace the picture */
@@ -377,5 +419,5 @@ Fl_GIF_Image::Fl_GIF_Image(const char *infname) : Fl_Pixmap((char *const*)0) {
 
 
 //
-// End of "$Id: Fl_GIF_Image.cxx 8864 2011-07-19 04:49:30Z greg.ercolano $".
+// End of "$Id$".
 //
