@@ -1,24 +1,15 @@
 //
-// "$Id: fl_images_core.cxx 5845 2007-05-20 00:01:06Z mike $"
+// "$Id$"
 //
 // FLTK images library core.
 //
-// Copyright 1997-2005 by Easy Software Products.
+// Copyright 1997-2010 by Easy Software Products.
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This library is free software. Distribution and use rights are outlined in
+// the file "COPYING" which should have been included with this file.  If this
+// file is missing or damaged, see the license at:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
+//     http://www.fltk.org/COPYING.php
 //
 // Please report all bugs and problems on the following page:
 //
@@ -53,10 +44,13 @@
 static Fl_Image	*fl_check_images(const char *name, uchar *header, int headerlen);
 
 
-//
-// 'fl_register_images()' - Register the image formats.
-//
-
+/**
+\brief Register the image formats.
+ *
+ This function is provided in the fltk_images library and 
+ registers all of the "extra" image file formats that are not part
+ of the core FLTK library.
+*/
 void fl_register_images() {
   Fl_Shared_Image::add_handler(fl_check_images);
 }
@@ -99,5 +93,5 @@ fl_check_images(const char *name,		// I - Filename
 
 
 //
-// End of "$Id: fl_images_core.cxx 5845 2007-05-20 00:01:06Z mike $".
+// End of "$Id$".
 //
