@@ -1,9 +1,9 @@
 /*
- * "$Id: numericsort.c 8864 2011-07-19 04:49:30Z greg.ercolano $"
+ * "$Id$"
  *
  * Numeric sorting routine for the Fast Light Tool Kit (FLTK).
  *
- * Copyright 1998-2010 by Bill Spitzak and others.
+ * Copyright 1998-2016 by Bill Spitzak and others.
  *
  * This library is free software. Distribution and use rights are outlined in
  * the file "COPYING" which should have been included with this file.  If this
@@ -29,20 +29,20 @@
 #    include <dirent.h>
 #  else
 #    define dirent direct
-#    if HAVE_SYS_NDIR_H
+#    ifdef HAVE_SYS_NDIR_H
 #      include <sys/ndir.h>
 #    endif /* HAVE_SYS_NDIR_H */
-#    if HAVE_SYS_DIR_H
+#    ifdef HAVE_SYS_DIR_H
 #      include <sys/dir.h>
 #    endif /* HAVE_SYS_DIR_H */
-#    if HAVE_NDIR_H
+#    ifdef HAVE_NDIR_H
 #      include <ndir.h>
 #    endif /* HAVE_NDIR_H */
 #  endif /* HAVE_DIRENT_H */
-#else /* For WIN32 variants */
-#  include <FL/filename.H>
 #endif /* !WIN32 || __CYGWIN__ */
 
+#include <FL/filename.H>
+
 /*
  * 'numericsort()' - Compare two directory entries, possibly with
  *                   a case-insensitive comparison...
@@ -98,5 +98,5 @@ int fl_numericsort(struct dirent **A, struct dirent **B) {
 }
 
 /*
- * End of "$Id: numericsort.c 8864 2011-07-19 04:49:30Z greg.ercolano $".
+ * End of "$Id$".
  */
