@@ -18,7 +18,7 @@
 
 #if !defined(WIN32) && !defined(__APPLE__)
 
-#include "../../FL/Xutf8.h"
+#include "../Xutf8.h"
 #include "imKStoUCS.c"
 
 long XKeysymToUcs(KeySym keysym) {
