@@ -1,5 +1,5 @@
 //
-// "$Id: checkers.cxx 8168 2011-01-02 03:55:23Z matt $"
+// "$Id$"
 //
 // Checkers game for the Fast Light Tool Kit (FLTK).
 //
@@ -8,20 +8,11 @@
 //
 // Copyright 1998-2010 by Bill Spitzak and others.
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This library is free software. Distribution and use rights are outlined in
+// the file "COPYING" which should have been included with this file.  If this
+// file is missing or damaged, see the license at:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
+//     http://www.fltk.org/COPYING.php
 //
 // Please report all bugs and problems on the following page:
 //
@@ -509,7 +500,7 @@ int fullexpand(node *f, int level) {
   node* n = f->son;
   if (!n->jump && n->brother) {if (level<1) return(1); level--;}
   int i;
-  node* sons[32]; for (i=0; (sons[i++] = n); n = n->brother);
+  node* sons[32]; for (i=0; (sons[i++] = n); n = n->brother) {/*empty*/}
   int ret = 1;
   for (i=0; ret && (n = sons[i++]);) {
     makemove(n);
@@ -1367,5 +1358,5 @@ int main(int argc, char **argv) {
 }
 
 //
-// End of "$Id: checkers.cxx 8168 2011-01-02 03:55:23Z matt $".
+// End of "$Id$".
 //
